#include <float.h>
#include <string>
#include <stdlib.h>
#include <string.h>
#include <memory.h>
#include <assert.h>
#include <stdio.h>
#include "StreamBuffer.h"
#include "xmath.h"

cStream::cStream()
{
	Attribute=0; lpBuffer=0;
	CurrentPos=CurrentSize=MaxSize=0;
}
cStream::cStream(char *fname,int attribute)
{
	Attribute=0; lpBuffer=0;
	CurrentPos=CurrentSize=MaxSize=0;
	open(fname,attribute);
}
cStream::cStream(int size,int attribute)
{
	Attribute=0; lpBuffer=0;
	CurrentPos=CurrentSize=MaxSize=0;
	open(size,attribute);
}
cStream::~cStream()
{
	close();
}
int cStream::open(char *fname,int attribute)
{
	close();
	return -1;
}
int cStream::open(int size,int attribute)
{
	close();
	Attribute=attribute;
	switch(Attribute&(STREAM_MEM|STREAM_IN|STREAM_OUT))
	{
		case STREAM_MEM|STREAM_IN:
		case STREAM_MEM|STREAM_OUT:
		case STREAM_MEM|STREAM_IN|STREAM_OUT:
			lpBuffer=new char[MaxSize=size];
			break;
		default:
			assert(0);
	}
	return 0;
}
int cStream::open(void *buf,int size,int attribute)
{
	close();
	Attribute=attribute;
	switch(Attribute&(STREAM_MEM|STREAM_IN|STREAM_OUT))
	{
		case STREAM_MEM|STREAM_IN:
		case STREAM_MEM|STREAM_OUT:
		case STREAM_MEM|STREAM_IN|STREAM_OUT:
			if(buf) { lpBuffer=(char*)buf; CurrentSize=MaxSize=size; }
			else lpBuffer=new char[MaxSize=size];
			break;
		default:
			assert(0);
	}
	return 0;
}
void cStream::close()			
{ 
	Attribute=0;
	if(lpBuffer) delete lpBuffer; lpBuffer=0;
	CurrentPos=CurrentSize=MaxSize=0;
}
int cStream::eof()
{ 
	if(lpBuffer&&CurrentPos<CurrentSize) return 0;
	return 1;
}
int cStream::length()
{ 
	if(lpBuffer) return CurrentSize;
	return 0;
}
void cStream::seek(int pos)
{
	CurrentPos=pos;
}
int cStream::write(const void *buf,int size)
{
	int NewPos=CurrentPos+size;
	if(NewPos<CurrentSize) 
	{
		memcpy(&lpBuffer[CurrentPos],buf,size);
		CurrentPos=NewPos;
	}
	else if(NewPos<MaxSize) 
	{
		memcpy(&lpBuffer[CurrentPos],buf,size);
		CurrentSize=CurrentPos=NewPos;
	}
	else
	{
		char *tmp=lpBuffer;
		if(size<MaxSize) lpBuffer=new char[MaxSize+=MaxSize];
		else lpBuffer=new char[MaxSize+=size];
		memcpy(lpBuffer,tmp,CurrentSize);
		memcpy(&lpBuffer[CurrentPos],buf,size);
		CurrentSize=CurrentPos=NewPos;
		delete tmp;
	}
	return size;
}
int cStream::read(void *buf,int size)
{
	int d=size-(CurrentSize-CurrentPos);
	if(d<=0) d=size;
	memcpy(buf,&lpBuffer[CurrentPos],d);
	CurrentPos+=d;
	return d;
}
char* cStream::str()
{
	return lpBuffer;
}
int cStream::getline(char *buf,int size)			
{ 
	assert(lpBuffer); 
	buf[0]=0;
	int count=0;
	while(CurrentPos<CurrentSize)
	{
		char ch=lpBuffer[CurrentPos++];
		if(ch!=' '&&ch!='\r'&&ch!='\t'&&ch!='\n') 
		{ buf[count++]=ch; break; }
	}
	while(CurrentPos<CurrentSize)
	{
		char ch=lpBuffer[CurrentPos++];
		if(ch=='\r'||ch=='\n') break;
		buf[count++]=ch;
		if(count>=size) return -1;
	}
	while(CurrentPos<CurrentSize)
	{
		char ch=lpBuffer[CurrentPos];
		if(ch==' '||ch=='\r'||ch=='\t'||ch=='\n') 
		{ ch=lpBuffer[CurrentPos++]; break; }
	}
	buf[count]=0;
	return count; 
}

cStream& cStream::operator << (float a)
{ 
	assert(lpBuffer); 
	char buf[256];
	sprintf(buf,"%f",a);
	write(buf,strlen(buf));
	return *this; 
}
cStream& cStream::operator << (double a)
{ 
	assert(lpBuffer); 
	char buf[256];
	sprintf(buf,"%f",a);
	write(buf,strlen(buf));
	return *this; 
}
cStream& cStream::operator << (const char *a)
{ 
	assert(lpBuffer); 
	write((void*)a,strlen(a));
	return *this; 
}
/*
cStream& cStream::operator >> (char &a)
{ 
	assert(lpBuffer); 
	char buf[256];
	(*this)>>buf;
	a=(char)atoi(buf);
	return *this; 
}
cStream& cStream::operator >> (unsigned char &a)
{ 
	assert(lpBuffer); 
	char buf[256];
	(*this)>>buf;
	a=(unsigned char)atoi(buf);
	return *this; 
}
cStream& cStream::operator >> (short &a)
{ 
	assert(lpBuffer); 
	char buf[256];
	(*this)>>buf;
	a=(short)atoi(buf);
	return *this; 
}
cStream& cStream::operator >> (unsigned short &a)
{ 
	assert(lpBuffer); 
	char buf[256];
	(*this)>>buf;
	a=(unsigned short)atoi(buf);
	return *this; 
}
cStream& cStream::operator >> (int &a)
{ 
	assert(lpBuffer); 
	char buf[256];
	(*this)>>buf;
	a=(int)atoi(buf);
	return *this; 
}
cStream& cStream::operator >> (unsigned int &a)
{ 
	assert(lpBuffer); 
	char buf[256];
	(*this)>>buf;
	a=(unsigned int)atoi(buf);
	return *this; 
}
cStream& cStream::operator >> (long &a)
{ 
	assert(lpBuffer); 
	char buf[256];
	(*this)>>buf;
	a=(long)atoi(buf);
	return *this; 
}
cStream& cStream::operator >> (unsigned long &a)
{ 
	assert(lpBuffer); 
	char buf[256];
	(*this)>>buf;
	a=(unsigned long)atoi(buf);
	return *this; 
}
cStream& cStream::operator >> (float &a)
{ 
	assert(lpBuffer); 
	char buf[256];
	(*this)>>buf;
	a=(float)StringToFloat(buf);
	return *this; 
}
cStream& cStream::operator >> (double &a)
{ 
	assert(lpBuffer); 
	char buf[256];
	(*this)>>buf;
	a=(double)StringToFloat(buf);
	return *this; 
}
cStream& cStream::operator >> (char *buf)
{
	assert(lpBuffer); 
	buf[0]=0;
	int count=0;
	while(CurrentPos<CurrentSize)
	{
		char ch=lpBuffer[CurrentPos++];
		if(ch!=' '&&ch!='\r'&&ch!='\t'&&ch!='\n') { buf[count++]=ch; break; }
	}
	while(CurrentPos<CurrentSize)
	{
		char ch=lpBuffer[CurrentPos++];
		if(ch!=' '&&ch!='\r'&&ch!='\t'&&ch!='\n') buf[count++]=ch;
		else break;
	}
	while(CurrentPos<CurrentSize)
	{
		char ch=lpBuffer[CurrentPos];
		if(ch==' '||ch=='\r'||ch=='\t'||ch=='\n') ch=lpBuffer[CurrentPos++];
		else break;
	}
	buf[count]=0;
	return *this; 
}
*/